@@ -1,11 +1,24 @@
-formatColumns = function(table, columns, template, ...) {
+formatColumns = function(table, columns, template, ..., appendTo = c('columnDefs', 'rowCallback')) {
+  if (!inherits(table, 'datatables'))
+    stop("Invalid table argument; a table object created from datatable() was expected")
   if (inherits(columns, 'formula')) columns = all.vars(columns)
   x = table$x
   colnames = base::attr(x, 'colnames', exact = TRUE)
   rownames = base::attr(x, 'rownames', exact = TRUE)
-  x$options$rowCallback = appendFormatter(
-    x$options$rowCallback, columns, colnames, rownames, template, ...
-  )
+  appendTo = match.arg(appendTo)
+  if (appendTo == 'columnDefs') {
+    x$options$columnDefs = append(
+      # must append to the front so that the later formatting
+      # can override the previous formatting
+      x$options$columnDefs, colFormatter(
+        columns, colnames, rownames, template, ...
+      ), after = 0L
+    )
+  } else {
+    x$options$rowCallback = appendFormatter(
+      x$options$rowCallback, columns, colnames, rownames, template, ...
+    )
+  }
   table$x = x
   table
 }
@@ -28,6 +41,8 @@ formatColumns = function(table, columns, template, ...) {
 #' @param before whether to place the currency symbol before or after the values
 #' @references See \url{https://rstudio.github.io/DT/functions.html} for detailed
 #'   documentation and examples.
+#' @note The length of arguments other than \code{table} should be 1 or the same as
+#'   the length of \code{columns}.
 #' @export
 #' @examples # !formatR
 #' library(DT)
@@ -97,12 +112,15 @@ formatSignif = function(
 #' @rdname formatCurrency
 #' @param method the method(s) to convert a date to string in JavaScript; see
 #'   \code{DT:::DateMethods} for a list of possible methods, and
-#'   \url{http://mzl.la/1xGe99W} for a full reference
+#'   \url{https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date}
+#'   for a full reference
 #' @param params a list parameters for the specific date conversion method,
 #'   e.g., for the \code{toLocaleDateString()} method, your browser may support
 #'   \code{params = list('ko-KR', list(year = 'numeric', month = 'long', day =
 #'   'numeric'))}
 formatDate = function(table, columns, method = 'toDateString', params = NULL) {
+  if (!inherits(table, 'datatables'))
+    stop("Invalid table argument; a table object created from datatable() was expected")
   x = table$x
   if (x$filter != 'none') {
     if (inherits(columns, 'formula')) columns = all.vars(columns)
@@ -149,74 +167,88 @@ formatStyle = function(
     fontWeight = fontWeight, color = color, backgroundColor = backgroundColor,
     background = background, ...
   ))
-  formatColumns(table, columns, tplStyle, valueColumns, match.arg(target), styles)
+  formatColumns(table, columns, tplStyle, valueColumns, match.arg(target), styles,
+                appendTo = 'rowCallback')
 }
 
 # turn character/logical indices to numeric indices
-name2int = function(name, names, rownames) {
+name2int = function(name, names, rownames, noerror = FALSE) {
   if (is.numeric(name)) {
     i = if (all(name >= 0)) name else seq_along(names)[name]
     if (!rownames) i = i - 1
     return(i)
   }
   i = unname(setNames(seq_along(names), names)[name]) - 1
-  if (any(is.na(i))) stop(
-    'You specified the columns: ', paste(name, collapse = ', '), ', ',
-    'but the column names of the data are ', paste(names, collapse = ', ')
-  )
+  if (any(is.na(i))) {
+    if (!noerror) stop(
+      'You specified the columns: ', paste(name, collapse = ', '), ', ',
+      'but the column names of the data are ', paste(names, collapse = ', ')
+    )
+    i = na.omit(i)
+  }
   i
 }
 
+colFormatter = function(name, names, rownames = TRUE, template, ...) {
+  i = name2int(name, names, rownames)
+  # see https://datatables.net/reference/option/columns.render
+  # #837 we only want to use the formatting for the "display" purpose
+  js = sprintf("function(data, type, row, meta) {
+    return type !== 'display' ? data : %s
+  }", template(...))
+  Map(function(i, js) list(targets = i, render = JS(js)), i, js, USE.NAMES = FALSE)
+}
+
 appendFormatter = function(js, name, names, rownames = TRUE, template, ...) {
   js = if (length(js) == 0) c('function(row, data) {', '}') else {
     unlist(strsplit(as.character(js), '\n'))
   }
-  i = name2int(name, names, rownames)
+  i = name2int(name, names, rownames, noerror = TRUE)
   JS(append(
     js, after = length(js) - 1,
     template(i, ..., names, rownames)
   ))
 }
 
-tplCurrency = function(cols, currency, interval, mark, digits, dec.mark, before, ...) {
+tplCurrency = function(currency, interval, mark, digits, dec.mark, before, ...) {
   sprintf(
-    "DTWidget.formatCurrency(this, row, data, %d, %s, %d, %d, %s, %s, %s);",
-    cols, jsValues(currency), digits, interval, jsValues(mark), jsValues(dec.mark),
+    "DTWidget.formatCurrency(data, %s, %d, %d, %s, %s, %s);",
+    jsValues(currency), digits, interval, jsValues(mark), jsValues(dec.mark),
     jsValues(before)
   )
 }
 
-tplString = function(cols, prefix, suffix, ...) {
+tplString = function(prefix, suffix, ...) {
   sprintf(
-    "DTWidget.formatString(this, row, data, %d, %s, %s);",
-    cols, jsValues(prefix), jsValues(suffix)
+    "DTWidget.formatString(data, %s, %s);",
+    jsValues(prefix), jsValues(suffix)
   )
 }
 
-tplPercentage = function(cols, digits, interval, mark, dec.mark, ...) {
+tplPercentage = function(digits, interval, mark, dec.mark, ...) {
   sprintf(
-    "DTWidget.formatPercentage(this, row, data, %d, %d, %d, %s, %s);",
-    cols, digits, interval, jsValues(mark), jsValues(dec.mark)
+    "DTWidget.formatPercentage(data, %d, %d, %s, %s);",
+    digits, interval, jsValues(mark), jsValues(dec.mark)
   )
 }
 
-tplRound = function(cols, digits, interval, mark, dec.mark, ...) {
+tplRound = function(digits, interval, mark, dec.mark, ...) {
   sprintf(
-    "DTWidget.formatRound(this, row, data, %d, %d, %d, %s, %s);",
-    cols, digits, interval, jsValues(mark), jsValues(dec.mark)
+    "DTWidget.formatRound(data, %d, %d, %s, %s);",
+    digits, interval, jsValues(mark), jsValues(dec.mark)
   )
 }
 
-tplSignif = function(cols, digits, interval, mark, dec.mark, ...) {
+tplSignif = function(digits, interval, mark, dec.mark, ...) {
   sprintf(
-    "DTWidget.formatSignif(this, row, data, %d, %d, %d, %s, %s);",
-    cols, digits, interval, jsValues(mark), jsValues(dec.mark)
+    "DTWidget.formatSignif(data, %d, %d, %s, %s);",
+    digits, interval, jsValues(mark), jsValues(dec.mark)
   )
 }
 
-tplDate = function(cols, method, params, ...) {
+tplDate = function(method, params, ...) {
   params = if (length(params) > 0) paste(',', toJSON(params)) else ''
-  sprintf("DTWidget.formatDate(this, row, data, %d, %s%s);", cols, jsValues(method), params)
+  sprintf("DTWidget.formatDate(data, %s%s);", jsValues(method), params)
 }
 
 DateMethods = c(
@@ -284,6 +316,9 @@ jsValues = function(x) {
 #' The function \code{styleColorBar()} can be used to draw background color bars
 #' behind table cells in a column, and the width of bars is proportional to the
 #' column values.
+#'
+#' The function \code{styleValue()} uses the column value as the CSS values.
+#'
 #' @param cuts a vector of cut points (sorted increasingly)
 #' @param values a vector of CSS values
 #' @export
@@ -326,10 +361,19 @@ styleEqual = function(levels, values, default = NULL) {
   for (i in seq_len(n)) {
     js = paste0(js, sprintf("value == %s ? %s : ", levels[i], values[i]))
   }
-  default = if (is.null(default)) 'value' else jsValues(default)
+  # set the css to null will leave the attribute as it is. Despite it's not
+  # documented explicitly but the jquery test covers this behavior
+  # https://github.com/jquery/jquery/commit/2ae872c594790c4b935a1d7eabdf8b8212fd3c3f
+  default = if (is.null(default)) 'null' else jsValues(default)
   JS(paste0(js, default))
 }
 
+#' @export
+#' @rdname styleInterval
+styleValue = function() {
+  JS('value')
+}
+
 #' @param data a numeric vector whose range will be used for scaling the
 #' table data from 0-100 before being represented as color bars. A vector
 #' of length 2 is acceptable here for specifying a range possibly wider or
@@ -345,7 +389,7 @@ styleColorBar = function(data, color, angle=90) {
   rg = range(data, na.rm = TRUE, finite = TRUE)
   r1 = rg[1]; r2 = rg[2]; r = r2 - r1
   JS(sprintf(
-    "isNaN(parseFloat(value)) || value <= %f ? '' : 'linear-gradient(%fdeg, transparent ' + (%f - value)/%f * 100 + '%%, %s ' + (%f - value)/%f * 100 + '%%)'",
+    "isNaN(parseFloat(value)) || value <= %f ? '' : 'linear-gradient(%fdeg, transparent ' + Math.max(%f - value, 0)/%f * 100 + '%%, %s ' + Math.max(%f - value, 0)/%f * 100 + '%%)'",
     r1, angle, r2, r, color, r2, r
   ))
 }
